#pragma once

#include <types/vertex.h>
#include <J3ML/Geometry.h>

enum class ColliderType : uint8_t {
    AxisAlignedBoundingBox = 0,
    OrientedBoundingBox = 1,
    CollisionMap = 2 //This one is the *ultra* slow one. Try not to use it super often.
};

using J3ML::Geometry::Triangle;

class Collider : public VertexArray {
public:
    ColliderType type;
    void draw() override;
    [[nodiscard]] std::vector<Triangle> getTriangles() const;

    //Useful for everything except AABB.
    Vector3 angle;
};

namespace Collision {
     Collider calculateOrientedBoundingBox(const VertexArray& vArray, const Vector3& angle);
     Collider calculateAxisAlignedBoundingBox(const VertexArray& vArray);
     
     bool vertexInsideAxisAlignedBoundingBox(const Vector3& position, const Collider& boundingBox);
     bool vertexInsideOrientedBoundingBox(const Vector3& position, const Collider& boundingBox);
     bool triangleVSAxisAlignedBoundingBoxCollides(const Triangle& triangle, const Collider& boundingBox);
     bool orientedBoundingBoxCollides(const Collider& boundingBox, const Collider& boundingBox2);
     bool axisAlignedBoundingBoxCollides(const Collider& boundingBox, const Collider& boundingBox2);
     bool collisionMapVSAxisAlignedBoundingBoxCollides(const Collider& boundingBox, const Collider& collisionMap);
}