#pragma once

#include <filesystem>
#include <cstdint>
#include <cstring>
#include <iostream>
#include <fstream>
#include <engine/world.h>
#include <rewindow/types/window.h>


enum class GAMESTATE: uint8_t {
    NORMAL = 0, //Gameplay.
    IN_MAIN_MENU = 1,
    IN_PAUSE_MENU = 2,
    IN_LEVEL_ANIMATION = 3, //A cutscene which moves entities in the world and cannot be interrupted.
};

enum class ENGINE_ERROR_CODE: uint8_t {
    NO_ERROR = 0,
    ANIM_OUT_OF_BOUNDS = 1,
    TEXTURE_NOT_FOUND = 2,
    SHADER_COMPILATION_ERROR = 3,
    SHADER_LINKING_ERROR = 4,
    INVALID_SHADER_TYPE = 5,
    SHADER_NOT_FOUND = 6,
    MODEL_NOT_FOUND = 7,
    MULTI_TEXTURE_SIZE_EXCEEDS = 8,
    TEXTURE_ERASED_WHILE_IN_USE = 9
};

struct EngineError {
    ENGINE_ERROR_CODE error;
    bool critical;

};
class Engine {
private:
    EngineError error;
public:
    std::string workingDir = std::filesystem::current_path().string();
    ReWindow::RWindow* window;
    World* world;
    void setError(ENGINE_ERROR_CODE code, bool critical);
    EngineError getError();
    //GAMESTATE gameState = GAMESTATE::NORMAL;
    //uint16_t minimumTickDelta = 15625;
    //float tickDelta = NULL;
    bool fullscreen;
    bool debug = true;
    bool useVBO = true;
    uint64_t tickCount = 0;
    uint64_t frameCount = 0;
    float frameDelta = 0;
    GLenum glError = GL_NO_ERROR;
    float nearPlane = 0.01f;
    float farPlane = 100.0f;
    float fov = 75;
    [[nodiscard]] float framerate() const;
    static void takeScreenshot() ;
    void quit() const;
    void quit (ENGINE_ERROR_CODE code);
    static float getGLVersion();
    void initGL();
    void loadConfig();

    void init();
    void preRender();
    void render();
    void postRender();
    void renderPass();
    void renderLoop();

    Engine() {
        //window = new(ReWindow::RWindow);
        //world = new(World);
        error = {ENGINE_ERROR_CODE::NO_ERROR, false};
    }
};

inline auto* engine = new(Engine);