#pragma once
#include <cstdint>
#include <types/vector.h>
#include <types/vertex.h>
#include <types/texture.h>
#include <J3ML/LinearAlgebra/Matrix4x4.h>
#include <J3ML/LinearAlgebra/Vector3.h>
#include <archive.h>
#include <engine/collision.h>

using J3ML::LinearAlgebra::Matrix4x4;
using J3ML::LinearAlgebra::Vector3;


class Entity {
private:
    virtual void loadTexture();
    void loadGeometry();
    GLfloat scale = 1.0f;
protected:
    std::string modelPath;
    std::string texturePath;
    Entity* parent;
    std::vector<Entity*> children;
public:
    std::string name;
    bool alive;


    virtual std::vector<std::string> GetEntityUUIDList() const {}

    template <class T>
    void SerializeMemberData(T& archive)
    {
        archive & uuid.c_str() & name.c_str() & alive;
    }

    template <class T>
    void Serialize(T& archive)
    {
        SerializeMemberData(archive);
        archive & GetEntityUUIDList();
    }

    Vector3 angle = {0,0,0}; //Pitch Yaw Roll, The orientation of the entity in the world,
    bool collidable = true; // Entities are solid by default.

    virtual Texture* getTexture();
    virtual VertexArray* getGeometry();
    virtual Collider getCollider();

    std::vector<Entity*> GetChildren();
    virtual void SetParent(Entity* parent);
    bool IsDescendantOf(Entity* ancestor);
    bool IsAncestorOf(Entity* descendant);
    std::vector<Entity*> GetDescendants();
    std::vector<Entity*> GetAncestors();
    virtual Entity* GetParent() const { return parent;}
    Entity* FindFirstChild(std::string search_name);
    Entity* GetFamilyTreeRoot() const;
    Entity *Add(Entity *newChild);
    void setScale(const float& multiplier);
    GLfloat getScale() const;

    // TODO: Constrain to DerivedEntityType
    template <typename T>
    T* FindFirstChildOfType() const
    {
        for (auto& child : children) {
            T* p = dynamic_cast<T>(child);

            if (p != nullptr)
                return p;
        }
        return nullptr;
    }

    void DescendantAdded(Entity* ent) {}
    void DescendantRemoved(Entity* ent) {}
    void AncestorAdded(Entity* ent) {}
    void AncestorRemoved(Entity* ent) {}


protected:
    Matrix4x4 coordinates;
    J3ML::LinearAlgebra::Position position; // X Y Z
public:
    std::string uuid;
    uint32_t ticksAlive; //At 64tps it'd take 776 days to overflow.
    [[nodiscard]] Vector3 GetPos() const;
    void SetPos(const Vector3& rhs);
    Matrix4x4 GetMatrix() const;
    void SetMatrix(const Matrix4x4& rhs);
    void erase();
    bool draw = true;
    bool occluded();
    bool isCollidingWith(Entity* entity);
    virtual void pre_render()  {}
    virtual void post_render() {}
    virtual void render();
    virtual void update(float elapsed) {}
    virtual void ticc(int tics)
    {
        ticksAlive++;

    }

    Entity();

    Entity(const Entity& rhs) = default; // Boilerplate: Copy Constructor
    Entity(Entity&& rhs)      = default; // Boilerplate: Move Constructor
    ~Entity() = default;
};


