#pragma once

#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <J3ML/LinearAlgebra/Vector3.h>
#include <J3ML/LinearAlgebra/Vector2.h>
#include <glad/glad.h>


using J3ML::LinearAlgebra::Vector3;
using J3ML::LinearAlgebra::Vector2;

typedef Vector3 Vertex;
typedef Vector2 TextureCoordinate;

struct FaceIndices {
	unsigned int vertexIndex[3];
	unsigned int texCoordIndex[3];
};

class VertexArray {
private:
    std::vector<Vertex> boundingVolume;
    void mapfaces(const FaceIndices& facedata, const std::vector<Vector3>& positions, const std::vector<Vector2>& uvs);
public:
    std::string name;
    std::vector<Vertex> vertices;
    std::vector<unsigned int> indices;
    std::vector<TextureCoordinate> texCoords;
    VertexArray getBoundingVolume(Vector3 angle);
    void rotate(const Vector3& angle);
    /// Loads data from an OBJ mesh file to populate the VertexArray
    /// @see LearnOpenGL::Mesh LearnOpenGL::Model J3ML::Geometry::TriangleMesh
    void load(const std::string& filename);
    static void erase(const VertexArray& vArray);
    virtual void draw();
    void drawWireframe();
    VertexArray(const std::string& filename);
    VertexArray() = default;

    GLuint vbo = 0; //Vertices
    GLuint ebo = 0; //Indices
    GLuint tbo = 0; //Texture Coordinates
};
