#include <engine/collision.h>
#include <limits>

Collider Collision::calculateOrientedBoundingBox(const VertexArray& vArray, const Vector3& angle) {
    Collider output;
    float minX = std::numeric_limits<float>::max();
    float minY = std::numeric_limits<float>::max();
    float minZ = std::numeric_limits<float>::max();
    float maxX = -std::numeric_limits<float>::max();
    float maxY = -std::numeric_limits<float>::max();
    float maxZ = -std::numeric_limits<float>::max();

    for (const auto& vertex : vArray.vertices) {
        if (vertex.x < minX)
            minX = vertex.x;
        if (vertex.y < minY)
            minY = vertex.y;
        if (vertex.z < minZ)
            minZ = vertex.z;

        if (vertex.x > maxX)
            maxX = vertex.x;
        if (vertex.y > maxY)
            maxY = vertex.y;
        if (vertex.z > maxZ)
            maxZ = vertex.z;
    }

    output.vertices.emplace_back(minX, minY, minZ);
    output.vertices.emplace_back(maxX, minY, minZ);
    output.vertices.emplace_back(minX, maxY, minZ);
    output.vertices.emplace_back(maxX, maxY, minZ);
    output.vertices.emplace_back(minX, minY, maxZ);
    output.vertices.emplace_back(maxX, minY, maxZ);
    output.vertices.emplace_back(minX, maxY, maxZ);
    output.vertices.emplace_back(maxX, maxY, maxZ);
    output.rotate(angle);

    output.type = ColliderType::OrientedBoundingBox;
    return output;
}

Collider Collision::calculateAxisAlignedBoundingBox(const VertexArray& vArray) {

    Collider output;
    float minX = std::numeric_limits<float>::max();
    float minY = std::numeric_limits<float>::max();
    float minZ = std::numeric_limits<float>::max();
    float maxX = -std::numeric_limits<float>::max();
    float maxY = -std::numeric_limits<float>::max();
    float maxZ = -std::numeric_limits<float>::max();

    for (const auto& vertex : vArray.vertices) {
        if (vertex.x < minX)
            minX = vertex.x;
        if (vertex.y < minY)
            minY = vertex.y;
        if (vertex.z < minZ)
            minZ = vertex.z;

        if (vertex.x > maxX)
            maxX = vertex.x;
        if (vertex.y > maxY)
            maxY = vertex.y;
        if (vertex.z > maxZ)
            maxZ = vertex.z;
    }
    output.vertices.emplace_back(minX, minY, minZ);
    output.vertices.emplace_back(maxX, minY, minZ);
    output.vertices.emplace_back(minX, maxY, minZ);
    output.vertices.emplace_back(maxX, maxY, minZ);
    output.vertices.emplace_back(minX, minY, maxZ);
    output.vertices.emplace_back(maxX, minY, maxZ);
    output.vertices.emplace_back(minX, maxY, maxZ);
    output.vertices.emplace_back(maxX, maxY, maxZ);

    output.type = ColliderType::AxisAlignedBoundingBox;
    return output;
}